/*
 * gsampler-channel-dialog.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 *
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <glib/gi18n-lib.h>

#include "gsampler-midi-inst-dialog.h"
#include "gsampler-option.h"
#include "gsampler-instrument.h"
#include "gsampler-utils.h"

G_DEFINE_TYPE (GsamplerMidiInstDialog, gsampler_midi_inst_dialog, GTK_TYPE_DIALOG);

static void 
gsampler_midi_inst_dialog_instrument_changed (GtkComboBox* combo, gpointer data) 
{
	gchar* name;
	GsamplerMidiInstDialog* self = GSAMPLER_MIDI_INST_DIALOG (data); 
	g_return_if_fail (self != NULL);
	g_return_if_fail (combo != NULL);

	if (!self->edit)
	{
		name = gsampler_text_combo_get_active (combo);
		if (name != NULL)
			gtk_entry_set_text (self->name_entry, name);
		g_free (name);
	}
}

static void 
gsampler_midi_inst_dialog_file_selection_changed (GtkFileChooser* button, gpointer data) 
{
	gchar* filename;
	gchar* ext;
	GsamplerInstrument* instrument;
	GsamplerMidiInstDialog* self = GSAMPLER_MIDI_INST_DIALOG (data); 
	g_return_if_fail (self != NULL);
	g_return_if_fail (button != NULL);

	if (self->update == 0)
	{
		filename = gtk_file_chooser_get_filename (button);
		if (filename == NULL) 
		{
			g_free (filename);
			return;
		}
		ext = gsampler_utils_get_extension (filename);
		if (g_strcmp0 (ext, "GIG") == 0) 
		{
			gtk_combo_box_set_active (self->engine, 0);
		}
		if (g_strcmp0 (ext, "SF2") == 0) 
		{
			gtk_combo_box_set_active (self->engine, 1);
		}
		if (g_strcmp0 (ext, "SFZ") == 0) 
		{
			gtk_combo_box_set_active (self->engine, 2);
		}
		instrument = gsampler_instrument_new ();
		if (gsampler_instrument_load_file (instrument, filename)) 
		{
			GSList* inst_list = NULL;

			gsampler_text_combo_remove_all (self->inst);
			inst_list = gsampler_instrument_get_list (instrument);
			while (inst_list) 
			{
				struct Struct_list_widget* instru;
				instru = (struct Struct_list_widget*) inst_list->data;
				gsampler_text_combo_insert (self->inst, (const gchar*) instru->Sdata, instru->Sid);
				inst_list = inst_list->next;
			}
			gtk_combo_box_set_active (self->inst, 0);
		}
		g_free (ext);
		g_free (filename);
	} else
		self->update--;
}

static void
gsampler_midi_inst_dialog_populate_create (GsamplerMidiInstDialog* self) 
{
	GsamplerOption* option;
	GtkTable* table;
	GtkFileFilter* filter;
	gchar* path;
	GtkScale* vol_scale;
	GtkLabel* engine_label;
	GtkLabel* file_label;
	GtkLabel* inst_label;
	GtkLabel* load_label;
	GtkLabel* name_label;
	GtkLabel* vol_label;
	GtkLabel* bank_label;
	GtkLabel* prog_label;
	gint i;
	GtkBox* vbox1;
	g_return_if_fail (self != NULL);

	option = gsampler_option_get_instance ();
	table = (GtkTable*) gtk_table_new (8, 2, TRUE);
	self->file = (GtkFileChooserButton*) gtk_file_chooser_button_new (_("Choose a sample file"),
	                                                                  GTK_FILE_CHOOSER_ACTION_OPEN);
	filter = gtk_file_filter_new ();
#if GTK_CHECK_VERSION (2, 90, 0)
	gtk_buildable_set_name ((GtkBuildable*) filter, "Sample");
#endif
	gtk_file_filter_add_pattern (filter, "*.gig");
	gtk_file_filter_add_pattern (filter, "*.GIG");
	gtk_file_filter_add_pattern (filter, "*.sf2");
	gtk_file_filter_add_pattern (filter, "*.SF2");
	gtk_file_filter_add_pattern (filter, "*.SFZ");
	gtk_file_filter_add_pattern (filter, "*.sfz");
	gtk_file_chooser_set_filter ((GtkFileChooser*) self->file, filter);
	path = gsampler_option_get_string (option, "Client", "sample-path");
	gtk_file_chooser_set_current_folder ((GtkFileChooser*) self->file, path);
	self->inst = gsampler_text_combo_create ();
	self->engine = gsampler_text_combo_create ();
	self->name_entry = (GtkEntry*) gtk_entry_new ();
	self->load_combo = gsampler_text_combo_create ();
	self->vol_adj = (GtkAdjustment*) gtk_adjustment_new (100, 0, 100, 1, 10, 0);
	vol_scale = (GtkScale*) gtk_hscale_new ( self->vol_adj);
	gtk_scale_set_value_pos (vol_scale, GTK_POS_RIGHT);
	self->bank_spin = (GtkSpinButton*) gtk_spin_button_new_with_range (0, 199, 1);
	self->prog_combo = gsampler_text_combo_create ();
	engine_label = (GtkLabel*) gtk_label_new (_("Select Engine :"));
	file_label = (GtkLabel*) gtk_label_new (_("Select File :"));
	inst_label = (GtkLabel*) gtk_label_new (_("Select Instrument :"));
	load_label = (GtkLabel*) gtk_label_new (_("Load Mode :"));
	name_label = (GtkLabel*) gtk_label_new (_("Midi Instrument Name :"));
	vol_label = (GtkLabel*) gtk_label_new (_("Volume :"));
	bank_label = (GtkLabel*) gtk_label_new (_("Midi Bank :"));
	prog_label = (GtkLabel*) gtk_label_new (_("Midi Program :"));
	gsampler_text_combo_append (self->load_combo, "Default");
	gsampler_text_combo_append (self->load_combo, "ON_DEMAND");
	gsampler_text_combo_append (self->load_combo, "ON_DEMAND_HOLD");
	gsampler_text_combo_append (self->load_combo, "PERSISTENT");
	gtk_combo_box_set_active (self->load_combo, 0);
	for (i=0; i < 128; i++) {
		gsampler_text_combo_append (self->prog_combo, g_strdup_printf ("%i", i));
	}
	gtk_combo_box_set_active (self->prog_combo, 0);
	gtk_table_attach_defaults (table, (GtkWidget*) file_label, 0, 1, 0, 1);
	gtk_table_attach_defaults (table, (GtkWidget*) self->file, 1, 2, 0, 1);
	gtk_table_attach_defaults (table, (GtkWidget*) inst_label, 0, 1, 1, 2);
	gtk_table_attach_defaults (table, (GtkWidget*) self->inst, 1, 2, 1, 2);
	gtk_table_attach_defaults (table, (GtkWidget*) engine_label, 0, 1, 2, 3);
	gtk_table_attach_defaults (table, (GtkWidget*) self->engine, 1, 2, 2, 3);
	gtk_table_attach_defaults (table, (GtkWidget*) load_label, 0, 1, 3, 4);
	gtk_table_attach_defaults (table, (GtkWidget*) self->load_combo, 1, 2, 3, 4);
	gtk_table_attach_defaults (table, (GtkWidget*) name_label, 0, 1, 4, 5);
	gtk_table_attach_defaults (table, (GtkWidget*) self->name_entry, 1, 2, 4, 5);
	gtk_table_attach_defaults (table, (GtkWidget*) vol_label, 0, 1, 5, 6);
	gtk_table_attach_defaults (table, (GtkWidget*) vol_scale, 1, 2, 5, 6);
	gtk_table_attach_defaults (table, (GtkWidget*) bank_label, 0, 1, 6, 7);
	gtk_table_attach_defaults (table, (GtkWidget*) self->bank_spin, 1, 2, 6, 7);
	gtk_table_attach_defaults (table, (GtkWidget*) prog_label, 0, 1, 7, 8);
	gtk_table_attach_defaults (table, (GtkWidget*) self->prog_combo, 1, 2, 7, 8);

	vbox1 = (GtkBox*) gtk_dialog_get_content_area ((GtkDialog*) self);
	gtk_box_pack_start (vbox1, (GtkWidget*) table, FALSE, TRUE, 0);
	g_signal_connect_object (self->inst, "changed", 
	                         (GCallback) gsampler_midi_inst_dialog_instrument_changed, self, 0);
	g_signal_connect_object ((GtkFileChooser*) self->file, "selection-changed", 
	                         (GCallback) gsampler_midi_inst_dialog_file_selection_changed, self, 0);
	gtk_widget_show_all ((GtkWidget*) self);
	g_free (path);
	g_object_unref (option);
}

static void
gsampler_midi_inst_dialog_populate_edit (GsamplerMidiInstDialog* self) 
{
	GtkTable* table;
	GtkScale* vol_scale;
	GtkLabel* engine_label;
	GtkLabel* inst_label;
	GtkLabel* load_label;
	GtkLabel* name_label;
	GtkLabel* vol_label;
	GtkLabel* file_label;
	GtkFileFilter* filter;
	gchar* path;
	GtkBox* vbox1;
	GsamplerOption *option;
	g_return_if_fail (self != NULL);

	option = gsampler_option_get_instance ();
	table = (GtkTable*) gtk_table_new (6, 2, TRUE);
	self->file = (GtkFileChooserButton*) gtk_file_chooser_button_new (_("Choose a sample file"),
	                                                                  GTK_FILE_CHOOSER_ACTION_OPEN);
	filter = gtk_file_filter_new ();
#if GTK_CHECK_VERSION (2, 90, 0)
	gtk_buildable_set_name ((GtkBuildable*) filter, "Sample");
#endif
	gtk_file_filter_add_pattern (filter, "*.gig");
	gtk_file_filter_add_pattern (filter, "*.GIG");
	gtk_file_filter_add_pattern (filter, "*.sf2");
	gtk_file_filter_add_pattern (filter, "*.SF2");
	gtk_file_filter_add_pattern (filter, "*.SFZ");
	gtk_file_filter_add_pattern (filter, "*.sfz");
	gtk_file_chooser_set_filter ((GtkFileChooser*) self->file, filter);
	path = gsampler_option_get_string (option, "Client", "sample-path");
	gtk_file_chooser_set_current_folder ((GtkFileChooser*) self->file, path);
	self->inst = gsampler_text_combo_create ();
	self->name_entry = (GtkEntry*) gtk_entry_new ();
	self->load_combo = gsampler_text_combo_create ();
	self->vol_adj = (GtkAdjustment*) gtk_adjustment_new (100, 0, 100, 1, 10, 0);
	vol_scale = (GtkScale*) gtk_hscale_new (self->vol_adj);
	gtk_scale_set_value_pos (vol_scale, GTK_POS_RIGHT);
	self->engine = gsampler_text_combo_create ();
	file_label = (GtkLabel*) gtk_label_new (_("Select File :"));
	engine_label = (GtkLabel*) gtk_label_new (_("Select Engine :"));
	inst_label = (GtkLabel*) gtk_label_new (_("Select Instrument :"));
	load_label = (GtkLabel*) gtk_label_new (_("Load Mode :"));
	name_label = (GtkLabel*) gtk_label_new (_("Midi Instrument Name :"));
	vol_label = (GtkLabel*) gtk_label_new (_("Volume :"));
	gsampler_text_combo_append (self->load_combo, "Default");
	gsampler_text_combo_append (self->load_combo, "ON_DEMAND");
	gsampler_text_combo_append (self->load_combo, "ON_DEMAND_HOLD");
	gsampler_text_combo_append (self->load_combo, "PERSISTENT");
	gtk_combo_box_set_active (self->load_combo, 0);

	gtk_table_attach_defaults (table, (GtkWidget*) file_label, 0, 1, 0, 1);
	gtk_table_attach_defaults (table, (GtkWidget*) self->file, 1, 2, 0, 1);
	gtk_table_attach_defaults (table, (GtkWidget*) inst_label, 0, 1, 1, 2);
	gtk_table_attach_defaults (table, (GtkWidget*) self->inst, 1, 2, 1, 2);
	gtk_table_attach_defaults (table, (GtkWidget*) engine_label, 0, 1, 2, 3);
	gtk_table_attach_defaults (table, (GtkWidget*) self->engine, 1, 2, 2, 3);
	gtk_table_attach_defaults (table, (GtkWidget*) load_label, 0, 1, 3, 4);
	gtk_table_attach_defaults (table, (GtkWidget*) self->load_combo, 1, 2, 3, 4);
	gtk_table_attach_defaults (table, (GtkWidget*) name_label, 0, 1, 4, 5);
	gtk_table_attach_defaults (table, (GtkWidget*) self->name_entry, 1, 2, 4, 5);
	gtk_table_attach_defaults (table, (GtkWidget*) vol_label, 0, 1, 5, 6);
	gtk_table_attach_defaults (table, (GtkWidget*) vol_scale, 1, 2, 5, 6);

	vbox1 = (GtkBox*) gtk_dialog_get_content_area ((GtkDialog*) self);
	gtk_box_pack_start (vbox1, (GtkWidget*) table, FALSE, TRUE, (guint) 0);

	/*g_signal_connect (self, "show",
	                  (GCallback) gsampler_midi_inst_dialog_realize, NULL);*/
	
	gtk_widget_show_all ((GtkWidget*) self);
	g_free (path);
	g_object_unref (option);
	self->update = 3;
}

static void 
gsampler_midi_inst_dialog_init (GsamplerMidiInstDialog * self)
{
	gtk_window_set_title ((GtkWindow*) self, _("Map a Midi Instrument"));
	gtk_container_set_border_width ((GtkContainer*) self,10);
	//gtk_window_set_default_size ((GtkWindow*) self, 300, 100);
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_APPLY, (gint) GTK_RESPONSE_APPLY);
	self->edit = FALSE;
	self->update = 0;
}


static void 
gsampler_midi_inst_dialog_finalize (GObject* obj) 
{
	G_OBJECT_CLASS (gsampler_midi_inst_dialog_parent_class)->finalize (obj);
}

static void 
gsampler_midi_inst_dialog_class_init (GsamplerMidiInstDialogClass * klass) 
{
	gsampler_midi_inst_dialog_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = gsampler_midi_inst_dialog_finalize;
}

GsamplerMidiInstDialog* 
gsampler_midi_inst_dialog_new (gboolean edit) {
	GsamplerMidiInstDialog* self = NULL;
	
	self = (GsamplerMidiInstDialog*) g_object_new (GSAMPLER_TYPE_MIDI_INST_DIALOG, NULL);
	if (!edit) 
	{
		gsampler_midi_inst_dialog_populate_create (self);
	} else
	{
		self->edit = TRUE;
		gsampler_midi_inst_dialog_populate_edit (self);
	}
	return self;
}

GsamplerMidiInstDialog* 
gsampler_midi_inst_dialog_create (void) 
{
	GsamplerMidiInstDialog* self = NULL;
	
	self = (GsamplerMidiInstDialog*) g_object_new (GSAMPLER_TYPE_MIDI_INST_DIALOG, NULL);
	gsampler_midi_inst_dialog_populate_create (self);
	return self;
}

GsamplerMidiInstDialog* 
gsampler_midi_inst_dialog_edit (void) 
{
	GsamplerMidiInstDialog* self = NULL;
	
	self = (GsamplerMidiInstDialog*) g_object_new (GSAMPLER_TYPE_MIDI_INST_DIALOG, NULL);
	gsampler_midi_inst_dialog_populate_edit (self);
	return self;
}

void
gsampler_midi_inst_dialog_connect (GsamplerMidiInstDialog *self)
{
	g_return_if_fail (self != NULL);

	//gint active = gtk_combo_box_get_active (self->inst);
	g_signal_connect_object (self->inst, "changed", 
	                         (GCallback) gsampler_midi_inst_dialog_instrument_changed, self, 0);
	g_signal_connect_object ((GtkFileChooser*) self->file, "selection-changed", 
	                         (GCallback) gsampler_midi_inst_dialog_file_selection_changed, self, 0);
	//gtk_combo_box_set_active (self->inst, active);
}