/*
 * gsampler-preference.c
 * Copyright (C) Lilian Monchalin 2011 <isaya07@users.sf.net>
 * 
 * gsampler is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * gsampler is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>

#include "gsampler-preference.h"
#include "gsampler-utils.h"
#include "gsampler-window.h"
#include "gsampler-option.h"
#include "gsampler-client-device.h"
#include "gsampler-client-channel.h"

#define UI_FILE "gsampler-preference.ui"

G_DEFINE_TYPE (GsamplerPreference, gsampler_preference, GTK_TYPE_DIALOG);

void
gsampler_preference_update_default_audio (GtkComboBox *widget, gpointer data)
{
	GsamplerOption *option = NULL;
	GtkTreeIter iter = {0};
	g_return_if_fail (widget != NULL);
	
	if (gtk_combo_box_get_active_iter (widget, &iter)) {
		GtkTreeModel* model;
		gchar* name = NULL;
		model = gtk_combo_box_get_model (widget);

		gtk_tree_model_get (model, &iter, 0, &name, -1);
		if (name != NULL)
		{
			option = gsampler_option_get_instance ();
			gsampler_option_set_string (option, "Client", "default-audio", name);
			gsampler_option_save_keyfile (option);
			g_object_unref (option);
		}
		g_free (name);
	}	
}

void
gsampler_preference_update_default_midi (GtkComboBox *widget, gpointer data)
{
	GsamplerOption *option = NULL;
	GtkTreeIter iter = {0};
	g_return_if_fail (widget != NULL);
	
	if (gtk_combo_box_get_active_iter (widget, &iter)) {
		GtkTreeModel* model;
		gchar* name = NULL;
		model = gtk_combo_box_get_model (widget);

		gtk_tree_model_get (model, &iter, 0, &name, -1);
		if (name != NULL)
		{
			option = gsampler_option_get_instance ();
			gsampler_option_set_string (option, "Client", "default-midi", name);
			gsampler_option_save_keyfile (option);
			g_object_unref (option);
		}
		g_free (name);
	}	
}

void
gsampler_preference_update_default_engine (GtkComboBox *widget, gpointer data)
{
	GsamplerOption *option = NULL;
	GtkTreeIter iter = {0};
	g_return_if_fail (widget != NULL);
	
	if (gtk_combo_box_get_active_iter (widget, &iter)) {
		GtkTreeModel* model;
		gchar* name = NULL;
		model = gtk_combo_box_get_model (widget);

		gtk_tree_model_get (model, &iter, 0, &name, -1);
		if (name != NULL)
		{
			option = gsampler_option_get_instance ();
			gsampler_option_set_string (option, "Client", "default-engine", name);
			gsampler_option_save_keyfile (option);
			g_object_unref (option);
		}
		g_free (name);
	}	
}

void
gsampler_preference_update_default_sample (GtkFileChooserButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gchar* name = NULL;
	g_return_if_fail (button != NULL);
	
	name = gtk_file_chooser_get_filename ((GtkFileChooser*) button);
	if (name != NULL)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "Client", "sample-path", name);
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
	g_free (name);
}

void
gsampler_preference_update_default_script (GtkFileChooserButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gchar* name = NULL;
	g_return_if_fail (button != NULL);

	name = gtk_file_chooser_get_filename ((GtkFileChooser*) button);
	if (name != NULL)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "Client", "script-path", name);
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
	g_free (name);
}

void
gsampler_preference_update_default_font (GtkFontButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gchar* name = NULL;
	g_return_if_fail (button != NULL);
	
	name = g_strdup ((gchar*) gtk_font_button_get_font_name ((GtkFontButton*) button));
	if (name != NULL)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "General", "font", name);
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
	g_free (name);	
}

void
gsampler_preference_update_default_exe (GtkFileChooserButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gchar* name = NULL;
	g_return_if_fail (button != NULL);

	name = gtk_file_chooser_get_filename ((GtkFileChooser*) button);
	if (name != NULL)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "Serveur", "serveur-com", name);
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
	g_free (name);
}

void
gsampler_preference_update_default_db (GtkFileChooserButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gchar* name = NULL;
	g_return_if_fail (button != NULL);

	name = gtk_file_chooser_get_filename ((GtkFileChooser*) button);
	if (name != NULL)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "Serveur", "db-path", name);
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
	g_free (name);
}

void
gsampler_preference_update_default_host (GtkEntry *entry, gpointer data)
{
	GsamplerOption *option = NULL;
	gchar* name = NULL;
	g_return_if_fail (entry != NULL);
	
	name = (gchar*) gtk_entry_get_text (entry);
	if (name != NULL)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "Serveur", "serveur-host", name);
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
	g_free (name);
}

void
gsampler_preference_update_default_port (GtkEntry *entry, gpointer data)
{
	GsamplerOption *option = NULL;
	gchar* name = NULL;
	g_return_if_fail (entry != NULL);
	
	name = (gchar*) gtk_entry_get_text (entry);
	if (name != NULL)
	{
		option = gsampler_option_get_instance ();
		gsampler_option_set_string (option, "Serveur", "serveur-port", name);
		gsampler_option_save_keyfile (option);
		g_object_unref (option);
	}
	g_free (name);
}

void
gsampler_preference_update_default_timeout (GtkSpinButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gint value;
	g_return_if_fail (button != NULL);
			
	value = gtk_spin_button_get_value_as_int (button);
	option = gsampler_option_get_instance ();
	gsampler_option_set_integer (option, "Serveur", "serveur-timeout", value);
	gsampler_option_save_keyfile (option);
	g_object_unref (option);
}

void
gsampler_preference_update_default_autostart (GtkCheckButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gboolean value;
	g_return_if_fail (button != NULL);

	value = gtk_toggle_button_get_active ((GtkToggleButton*) button);
	option = gsampler_option_get_instance ();
	gsampler_option_set_boolean (option, "Serveur", "serveur-autostart", value);
	gsampler_option_save_keyfile (option);
	g_object_unref (option);
}

void
gsampler_preference_update_default_wait (GtkSpinButton *button, gpointer data)
{
	GsamplerOption *option = NULL;
	gint value;
	g_return_if_fail (button != NULL);
			
	value = gtk_spin_button_get_value_as_int (button);
	option = gsampler_option_get_instance ();
	gsampler_option_set_integer (option, "Serveur", "serveur-wait", value);
	gsampler_option_save_keyfile (option);
	g_object_unref (option);
}

static GObject *
gsampler_preference_constructor (GType type, guint n_construct_properties, 
                               	 GObjectConstructParam * construct_properties)
{
	GObject * obj = NULL;
	GsamplerPreference * self = NULL;
	GtkWidget *dialog_box = NULL;
	GtkNotebook *notebook = NULL;
	GtkComboBox* audio_combo = NULL;
	GtkComboBox* midi_combo = NULL;
	GtkComboBox* engine_combo = NULL;
	GtkFileChooserButton* sample_chooser = NULL;
	GtkFileChooserButton* script_chooser = NULL;
	GtkFileChooserButton* db_chooser = NULL;
	GtkFileChooserButton* com_chooser = NULL;
	GtkFontButton* font_button = NULL;
	GtkEntry* host_entry = NULL;
	GtkEntry* port_entry = NULL;
	GtkSpinButton* time_spin = NULL;
	GtkCheckButton* autostart_check = NULL;
	GtkSpinButton* wait_spin = NULL;

	GtkWindow* mainwin = NULL;
	GsamplerOption* option = NULL;
	GsamplerClient* client = NULL;
	gchar** drivers = NULL;
	gchar* defaul = NULL;
	gint driver_it;
	gint i;
	gint drivers_lenght = 0;
	gchar *load_file = NULL;
	GtkBuilder *builder = NULL;
	GError* error = NULL;
	
	obj = G_OBJECT_CLASS (gsampler_preference_parent_class)->constructor (type, n_construct_properties, construct_properties);
	self = GSAMPLER_PREFERENCE (obj);
	mainwin = (GtkWindow*) gsampler_main_window_get_instance ();
	gtk_window_set_transient_for ((GtkWindow*) self, mainwin);
	gtk_dialog_add_button ((GtkDialog*) self, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE);
	dialog_box = gtk_dialog_get_content_area ((GtkDialog*) self);

	load_file = gsampler_utils_get_data_file (UI_FILE);
	if (load_file != NULL)
	{
		builder = gtk_builder_new ();
		gtk_builder_add_from_file (builder, load_file, &error);
		if (error != NULL)
		{
			g_warning ("Couldn't load builder file: %s", error->message);
			g_error_free (error);
		}	
	
		notebook = GTK_NOTEBOOK (gtk_builder_get_object (builder, "notebook"));
		audio_combo = GTK_COMBO_BOX (gtk_builder_get_object (builder, "audio_combo"));
		midi_combo = GTK_COMBO_BOX (gtk_builder_get_object (builder, "midi_combo"));
		engine_combo = GTK_COMBO_BOX (gtk_builder_get_object (builder, "engine_combo"));
		sample_chooser = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (builder, "sample_chooser"));
		script_chooser = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (builder, "script_chooser"));
		font_button = GTK_FONT_BUTTON (gtk_builder_get_object (builder, "font_button"));
		db_chooser = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (builder, "db_chooser"));
		com_chooser = GTK_FILE_CHOOSER_BUTTON (gtk_builder_get_object (builder, "com_chooser"));
		host_entry = GTK_ENTRY (gtk_builder_get_object (builder, "host_entry"));
		port_entry = GTK_ENTRY (gtk_builder_get_object (builder, "port_entry"));
		time_spin = GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "time_spin"));
		autostart_check = GTK_CHECK_BUTTON (gtk_builder_get_object (builder, "autostart_check"));
		wait_spin = GTK_SPIN_BUTTON (gtk_builder_get_object (builder, "wait_spin"));
	
		gtk_box_pack_start ((GtkBox*) dialog_box, (GtkWidget*) notebook, TRUE, TRUE, 5);
	
		g_object_unref (builder);
		g_free (load_file);
		
		/* populate the widget */
		option = gsampler_option_get_instance ();
		if (gsampler_man_window_status ()) {
			client = gsampler_client_get_instance ();
			gsampler_text_combo_configure (audio_combo);
			defaul = gsampler_option_get_string (option, "Client", "default-audio");
			drivers = gsampler_client_get_audio_driver_list (client);
			if (drivers != NULL)
				drivers_lenght = g_strv_length (drivers);
			i = 0;
			for (driver_it = 0; driver_it < drivers_lenght; driver_it = driver_it + 1) {
				gchar* driver;
				driver = g_strdup (drivers[driver_it]);
				if (g_strcmp0 (driver, "NONE") == 0) {
					g_free (driver);
					break;
				}
				gsampler_text_combo_append (audio_combo, driver);
				if (g_strcmp0 (driver, defaul) == 0) {
					gtk_combo_box_set_active (audio_combo, i);
				}
				i++;
				g_free (driver);		
			}
	
			g_strfreev (drivers);
			g_free (defaul);
			drivers = NULL;
			drivers_lenght = 0;
			gsampler_text_combo_configure (midi_combo);
			defaul = gsampler_option_get_string (option, "Client", "default-midi");
			drivers = gsampler_client_get_midi_driver_list (client);
			if (drivers != NULL)
				drivers_lenght = g_strv_length (drivers);
			i = 0;
			for (driver_it = 0; driver_it < drivers_lenght; driver_it = driver_it + 1) {
				gchar* driver;
				driver = g_strdup (drivers[driver_it]);
				if (g_strcmp0 (driver, "NONE") == 0) {
					g_free (driver);
					break;
				}
				gsampler_text_combo_append (midi_combo, driver);
				if (g_strcmp0 (driver, defaul) == 0) {
					gtk_combo_box_set_active ((GtkComboBox*) midi_combo, i);
				}
				i++;
				g_free (driver);
			}
			g_strfreev (drivers);
			g_free (defaul);
			drivers = NULL;
			drivers_lenght = 0;
			gsampler_text_combo_configure (engine_combo);
			defaul = gsampler_option_get_string (option, "Client", "default-engine");
			drivers = gsampler_client_get_available_engine_list (client);
			if (drivers != NULL)
				drivers_lenght = g_strv_length (drivers);
			i = 0;
			for (driver_it = 0; driver_it < drivers_lenght; driver_it = driver_it + 1) {
				gchar* driver;
				driver = string_replace (drivers[driver_it], "'", "");
				if (g_strcmp0 (driver, "NONE") == 0) {
					g_free (driver);
					break;
				}
				gsampler_text_combo_append (engine_combo, driver);
				if (g_strcmp0 (driver, defaul) == 0) {
					gtk_combo_box_set_active ((GtkComboBox*) engine_combo, i);
				}
				i++;
				g_free (driver);
			}
			g_free (defaul);
			g_strfreev (drivers);
			g_object_unref (client);
		} else 
		{
			gtk_widget_set_sensitive ((GtkWidget*) audio_combo, FALSE);
			gtk_widget_set_sensitive ((GtkWidget*) midi_combo, FALSE);
			gtk_widget_set_sensitive ((GtkWidget*) engine_combo, FALSE);
		}
		gtk_file_chooser_set_current_folder ((GtkFileChooser*) sample_chooser, 
			                                 gsampler_option_get_string (option, "Client", "sample-path"));
		gtk_file_chooser_set_current_folder ((GtkFileChooser*) script_chooser, 
			                                 gsampler_option_get_string (option, "Client", "script-path"));
		gtk_font_button_set_font_name (font_button, 
			                           gsampler_option_get_string (option, "General", "font"));
		gtk_file_chooser_set_filename ((GtkFileChooser*) com_chooser,
			                           gsampler_option_get_string (option, "Serveur", "serveur-com"));
		gtk_file_chooser_set_filename ((GtkFileChooser*) db_chooser,
			                           gsampler_option_get_string (option, "Serveur", "db-path"));
		gtk_entry_set_text (host_entry, 
			                gsampler_option_get_string (option, "Serveur", "serveur-host"));
		gtk_entry_set_text (port_entry, 
			                gsampler_option_get_string (option, "Serveur", "serveur-port"));
		gtk_spin_button_set_value (time_spin, 
			                       gsampler_option_get_integer (option, "Serveur", "serveur-timeout"));
		gtk_spin_button_set_value (wait_spin, 
			                       gsampler_option_get_integer (option, "Serveur", "serveur-wait"));
		gtk_toggle_button_set_active ((GtkToggleButton*) autostart_check, 
			                          gsampler_option_get_boolean (option, "Serveur", "serveur-autostart"));


		g_signal_connect (audio_combo, "changed", 
		                  (GCallback) gsampler_preference_update_default_audio, NULL);
		g_signal_connect (midi_combo, "changed", 
		                  (GCallback) gsampler_preference_update_default_midi, NULL);
		g_signal_connect (engine_combo, "changed", 
		                  (GCallback) gsampler_preference_update_default_engine, NULL);
		g_signal_connect (sample_chooser, "file-set", 
		                  (GCallback) gsampler_preference_update_default_sample, NULL);
		g_signal_connect (script_chooser, "file-set", 
		                  (GCallback) gsampler_preference_update_default_script, NULL);
		g_signal_connect (font_button, "font-set", 
		                  (GCallback) gsampler_preference_update_default_font, NULL);
		g_signal_connect (autostart_check, "toggled", 
		                  (GCallback) gsampler_preference_update_default_autostart, NULL);
		g_signal_connect (wait_spin, "value-changed", 
		                  (GCallback) gsampler_preference_update_default_wait, NULL);
		g_signal_connect (com_chooser, "file-set", 
		                  (GCallback) gsampler_preference_update_default_exe, NULL);
		g_signal_connect (db_chooser, "file-set", 
		                  (GCallback) gsampler_preference_update_default_db, NULL);
		g_signal_connect (host_entry, "activate", 
		                  (GCallback) gsampler_preference_update_default_host, NULL);
		g_signal_connect (port_entry, "activate", 
		                  (GCallback) gsampler_preference_update_default_port, NULL);
		g_signal_connect (time_spin, "value-changed", 
		                  (GCallback) gsampler_preference_update_default_timeout, NULL);
		
		
		
		g_signal_connect_swapped ((GtkDialog*) self, "response", 
			              (GCallback) gtk_widget_destroy, self);
		
		g_object_unref (option);

		
	}
	g_object_unref (mainwin);
	return obj;
}

static void 
gsampler_preference_init (GsamplerPreference * self) 
{

}

static void 
gsampler_preference_finalize (GObject* obj) {
	G_OBJECT_CLASS (gsampler_preference_parent_class)->finalize (obj);
}

static void 
gsampler_preference_class_init (GsamplerPreferenceClass * klass) {
	gsampler_preference_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = gsampler_preference_finalize;
	G_OBJECT_CLASS (klass)->constructor = gsampler_preference_constructor;
}

GsamplerPreference* 
gsampler_preference_new (void) {
	return g_object_new (GSAMPLER_TYPE_PREFERENCE, NULL);
}